#!/usr/bin/env perl
# (c) Deep IT, Uncle MiF. 2003-2010. mac_support@realsofts.com
# DictBuilder Utilities framework
package DBUtil;
use Carp qw(carp);

sub import
{
	my $class = shift;
	my $caller = caller(0);
	*{$caller."::readFile"} = \&readFile;
	*{$caller."::writeFile"} = \&writeFile;
	*{$caller."::safeExec"} = \&pipeChain;
}
sub readFile
{
	my $fName = shift;

	local * F;
	local $/;
	undef $/;
	my $ok = 1;
	my $res = '';
	open(F,"<$fName") or $ok = 0;
	if ($ok)
	{
		$res = <F>;
		close(F);
	}
	return $res;
}

sub writeFile
{
	my ($fName,$data) = @_;

	local * O;
	my $ok = 1;
	open(O,">$fName") or $ok = 0;
	if ($ok)
	{
		print O $data;
		close(O);
		return 1;
	}
	return 0;
}

use POSIX qw(:sys_wait_h);

sub pipeChain
{
	my @chain = @_;
	my $data = '';
	my $child;
	foreach my $process (@chain)
	{
		if (ref($process) eq '')
		{
			die "already has initial data\n" if ($data);
			$data = $process;
			next;
		}
		my ($command,@args) = @{$process};
		if ($command)
		{
			local *IN_IN, *IN_OUT;
			pipe IN_IN, IN_OUT;
			local *OUT_IN, *OUT_OUT;
			pipe OUT_IN, OUT_OUT;
			#print STDERR "executing: $command\n";
			$child = fork();
			if ($child == 0)# child
			{
				close IN_OUT; close OUT_IN;
				open STDIN,'<&IN_IN'; open STDOUT,'>&OUT_OUT';
				exec $command, @args or die "Can't run $command\n";# will read from IN_IN and out to OUT_OUT
			}
			else
			{
				close(IN_IN); close(OUT_OUT);
				if (defined $data)
				{
					print IN_OUT $data;
					undef $data;
				}
				close IN_OUT;
				while(<OUT_IN>)
				{
					$data .= $_;
				}
				waitpid($child,WNOHANG);
				close(OUT_IN);
			}
		}
	}
	return $data if ($child);
}

=for developer: demo

my $txtfile = 'test.txt';
my $opt = '-l';

print "{\n" . 
pipeChain(
['head','-10',$txtfile],
['wc',$opt]
)
. "}\n";

# or

print pipeChain('hello',['wc','-c']);

=cut

1;
