#!/usr/bin/env perl
# (c) Deep IT, Uncle MiF, 2003-2010, mac_support@realsofts.com

require 5.8.0;
use strict;

############### customize options  here ##########################
# resources rel from .xdxf file
my $sRel = '.';# rel path to dict sounds
my $iRel = '.';# rel path to dict images
my $vRel = '.';# rel path to dict videos

# fonts
my $cKey = ' <font class=ckey>%s</font><br> ';
my $cOpt = '<font class=copt>%s</font>';
my $cDef = ' <font class=cdef>%s</font><br> ';
my $cPos = ' <font class=cpos>%s</font> ';
my $cTense = ' <font class=ctense>%s</font> ';
my $cDirectTranslation = ' <font class=cdirecttranslation>%s</font> ';
my $cAbr = ' <font class=cabr>%s</font> ';

my $cTranscription = ' <font class=ctranscription>%s</font> ';
my $cCommentary = ' <font class=ccommentary>%s</font><br> ';
my $cExample = ' <font class=cexample>%s</font><br> ';
my $cColor = '<font class=ccolor>%s</font>';
my $cDirectColor = '<font color=%s class=cdirectcolor>%s</font>';

my $cMultimedia = ' <font class=cmultimedia>%s</font> ';
my $cImage = '<img src="__E__/Images/%s">';
my $cSound = '<object type="video/quicktime" data="__E__/Sounds/%s"><param name="controller" value="true" /></object>';
my $cVideo = '<a href="__E__/Movies/%s">%s</a>';

###################################################################

our $modules;
our $xdxf;
BEGIN
{ 
 select STDIN;
 $| = 1;
 select STDOUT;
 $| = 1;
 $modules = __FILE__;
	$modules = '.' if ($modules !~ /\//);
 $modules =~ s/\/[^\/]*$//gos;
	my $pmodules = $modules;
	$pmodules =~ s/'/\\'/gos;
 eval "use lib ('$pmodules');";
}
use lib qw(.);
use Localization::Localization;
use DBUtil;
$xdxf = readFile("$modules/xdxf.css");

$| = 1;

my $dname = $ARGV[0];
my $out = $ARGV[1];

my $verPath = $0;
$verPath =~ s/\/[^\/]*$//gos;
$verPath .= '/version';
my $ver = readFile($verPath);
$ver =~ s/\s+$//gos;

print "\n";
print _L("(c) MiF, TranslateIt!, XDXF file parser version %#.\n\n",$ver);
unless($dname)
{
	print _L("Usage: %# <dictFile> [<outDir>]\n",$0);
	print _L("\nExample: %# dict.xdxf Fr_Ru_dictDir\n\n",$0);
	exit(1);
}
unless(-f $dname)
{
	print _L("File not found: %#\n",$dname);
	exit(2);
}

my $lastPathComponent = $dname;
$lastPathComponent =~ s/\.xdxf$//i;
$lastPathComponent =~ s/^.*\///gos;
if (!$lastPathComponent || $lastPathComponent =~ /^\.+$/)
{
	 print _L("Can't parse properly dictionary path\n");
		 exit(20);
}

$out ||= './Unpacked Dictionaries/';

my $silent = $ARGV[2];
$silent ||= 0;
$silent = ($silent =~ /^silent$/i) ? 1 : 0;

$out .= '/' if ($out !~ /\/$/);
$out .= $lastPathComponent;
my $in = $dname;
$in = '.' if ($in !~ /\//);
$in =~ s/\/[^\/]*$//gso;
$in = '.' unless ($in);

print _L("Using file: '%#'\n",$dname);
print _L("Input directory: '%#'\n",$in);

my $testHeader = safeExec(['head', '-10', $dname]);
if ($testHeader !~ /<\?xml\s+version\s*=\s*['"]1.0['"]\s+encoding\s*=\s*['"]UTF-8['"]\s*\?>/si)
{
	print _L("XML header must be present with encoding UTF-8\n");
	exit(101);
}

if ($testHeader !~ /<xdxf\s+lang_from\s*=\s*['"].+?['"]\s+lang_to\s*=\s*['"].+?['"]\s+format\s*=\s*['"]visual['"]\s*>/si)
{
	print _L("XDXF file header must be present with 'visual' format specified\n");
	exit(102);
}

print _L("XDXF checked...\n");

print _L("Parsing '%#' to directory '%#'\n",$dname,$out);

if (-d $out || -f $out)
{
	print _L("Directory/file '%#' already exists. Remove now [N/yes]? ",$out);
	my $answer = <STDIN>;
	$answer =~ s/\s+$//gos;
	if ($answer !~ /^yes$/i)
	{
		print _L("User canceled (you must type YES to continue)...\n");
		exit(3);
	}
}

system('rm', '-rf', $out); system('mkdir', '-p', $out);

my $words = "$out/Words";
my $replacements = "$out/Replacements";
my $extensions = "$out/Extensions";
my $images = "$extensions/Images";
my $sounds = "$extensions/Sounds";
my $videos = "$extensions/Movies";

print _L("Words directory will be: %#\n",$words);
print _L("Replacements directory will be: %#\n",$replacements);
print _L("Extensions directory will be: %#\n",$extensions);
print _L("Sounds directory will be: %#\n",$sounds);
print _L("Images directory will be: %#\n",$images);
print _L("Movies directory will be: %#\n",$videos);

my $css = "$extensions/css";

system('mkdir', '-p', $words); system('mkdir', '-p', $replacements); system('mkdir', '-p', $extensions);
system('mkdir', '-p', $sounds); system('mkdir', '-p', $images); system('mkdir', '-p', $videos);
system('mkdir', '-p', $css); writeFile("$css/xdxf.css",$xdxf);

print _L("Processing dictionary file...\n");

local * DICT;

my $ok = 1;
open (DICT, "<$dname") or $ok = 0;
unless($ok)
{
	print _L("Can't open %# for reading", $dname);
	exit(4);
}

print _L("Loading XDXF file to memory...\n");
my $cards = 0;
my $file = '';
{
	local $/ = undef;
	$file = <DICT>;
}
close(DICT);

print _L("Done.\nParsing entries...\n\n");

while ($file =~ /<ar(?:\s+[^>]+)?>(.*?)<\/ar>/gios)
{
	$cards++;
	parseCard($1);
}

sub mark
{
	my ($str, $as, $color) = @_;
	if ($as eq 'direct translation')
	{
		return sprintf $cDirectTranslation, $str;
	}	
	elsif ($as eq 'transcription')
	{
		return sprintf $cTranscription, $str;
	}
	elsif ($as eq 'key')
	{
		return sprintf $cKey, $str;
	}
	elsif ($as eq 'opt')
	{
		return sprintf $cOpt, $str;
	}
	elsif ($as eq 'def')
	{
		return sprintf $cDef, $str;
	}
	elsif ($as eq 'pos')
	{
		return sprintf $cPos, $str;
	}
	elsif ($as eq 'tense')
	{
		return sprintf $cTense, $str;
	}
	elsif ($as eq 'example')
	{
		return sprintf $cExample, $str;
	}
	elsif ($as eq 'commentary')
	{
		return sprintf $cCommentary, $str;
	}
	elsif ($as eq 'abr')
	{
		return sprintf $cAbr, $str;
	}
	elsif ($as eq 'color')
	{
		return sprintf $cColor, $str;
	}
	elsif ($as eq 'multimedia')
	{
		my $spath;
		if ($str =~ /\.(?:png|bmp|pcx|dcx|jpg|tiff?|gif)$/soi)
		{
			# image
			$spath = "$in/$iRel/$str";
			return '' if (system('cp', $spath, $images));
			return sprintf $cImage, $str;
		}
		elsif ($str =~ /\.(?:wav|au|mp3|mp4)$/soi)
		{
			# sound
			$spath = "$in/$sRel/$str";
			return '' if (system('cp', $spath, $sounds));
			return sprintf $cSound, $str;
		}
		elsif ($str =~ /\.(?:avi|mov|mpg)/soi)
		{
			# movie
			$spath = "$in/$vRel/$str";
			return '' if (system('cp', $spath, $videos));
			return sprintf $cVideo, $str, $str;
		}
		else
		{
			# without extension hack
			if ($str !~ /\./os)
			{
				# assuming WAV, but first of all we test converted MP3 file
				$spath = "$in/$sRel/$str.mp3";
				if (-f $spath)
				{
					return mark($str . '.mp3', 'multimedia');
				}
				$spath =~ s/mp3$/wav/og;
				if (-f $spath)
				{
					return mark($str . '.wav', 'multimedia');
				}
			}
		}
		return sprintf $cMultimedia, $str;
	}
	elsif ($as eq 'direct color')
	{
		return sprintf $cDirectColor, $color, $str;
	}
	return $str;
}

sub parseCard
{
	my $article = shift;

	my $cardWord;
	if ($article =~ /<k>(.+?)<\/k>/gosi)
	{
		$cardWord = $1;

		$cardWord =~ s/^\s+//gos;
		$cardWord =~ s/\s+$//gos;
	}
	else
	{
		return;
	}

	my ($word, $article) = ($cardWord, $article);
	$word =~ s/<nu>.*?<\/nu>//gosi;
	$word =~ s/<.+?>//gosi;
	$word =~ s/^\s+//gos;
	$word =~ s/\s+$//gos;
	$word =~ s/&lt;/</gos;
	$word =~ s/&gt;/>/gos;
	$word =~ s/&quot;/"/gos;
	$word =~ s/&apos;/'/gos;

	$word =~ s/&amp;/&/gos;

	return unless ($word ne '' && $article ne '');

	{

		$article =~ s/<k>(.+?)<\/k>/mark($1,'key')/gosei;# key
		$article =~ s/<opt>(.+?)<\/opt>/mark($1,'opt')/gosei;# optional key part
		$article =~ s/<def>(.+?)<\/def>/mark($1,'def')/gosei;# def
		$article =~ s/<def\s+l\s*=\s*['"](.*?)['"]\s*>(.+?)<\/def>/mark("$1 $2",'def')/gosei;# def
		$article =~ s/<dtrn>(.+?)<\/dtrn>/mark($1,'direct translation')/gosei;# direct translation
		$article =~ s/<pos>(.+?)<\/pos>/mark($1,'pos')/gosei;# pos
		$article =~ s/<tense>(.+?)<\/tense>/mark($1,'tense')/gosei;# tense
		$article =~ s/<abr>(.+?)<\/abr>/mark($1,'abr')/gosei;# abbriviation
		$article =~ s/<tr>(.+?)<\/tr>/mark($1,'transcription')/gosei;# transcription
		$article =~ s/<co>(.+?)<\/co>/mark($1,'commentary')/gosei;# commentary
		$article =~ s/<ex>(.+?)<\/ex>/mark($1,'example')/gosei;# example
		$article =~ s/<c>(.+?)<\/c>/mark($1,'color')/gosei;# color mark
		
		$article =~ s/<iref>\s*(.+?)\s*<\/iref>/<a href="$1">$1<\/a>/gosi;# internet resource ref
		$article =~ s/<iref\s+href=['"](.+?)['"]\s*>(.+?)<\/iref>/<a href="$1">$2<\/a>/gosi;# internet resource ref
		$article =~ s/<kref>\s*(?:<k>)?(.+?)(?:<\/k>)?\s*<\/kref>/[:[$1]:]/gosi;# ref to another key phrase
		$article =~ s/<c\s+c\s*=\s*['"]([^\>'"]+)['"]\s*>(.+?)<\/c>/mark($2,'direct color',$1)/gosei;# color as specified

		$article =~ s/<rref(?:\s+.*?)?>(.+?)<\/rref>/mark($1,'multimedia')/gosei;# multimedia resource ref		

		$article = "<link type=text/css rel=stylesheet href=\"__E__/css/xdxf.css\" />$article";

		local * ART;
		my $ok = 1;
 	open (ART, ">$words/$word-$cards.html") or $ok = 0;
		if ($ok)
		{
			print ART $article;
			close(ART);
		}
		else
		{
			$ok = 1;
			open (ART, ">$replacements/_REPLACEMENT_$cards.html") or $ok = 0;
			if ($ok)
			{
				print _L("Using replacement file for word '%#'\n",$word);
				print ART $word;
				close(ART);
				$ok = 1;
				open (ART, ">$words/_REPLACEMENT_$cards.html") or $ok = 0;
				if ($ok)
				{
					print ART $article;
					close(ART);
				}
				else
				{
					print L("Failed: Something wrong with word '%#' and replacement _REPLACEMENT_%#\n", $word, $cards);
				}
			}
			else
			{
				print _L("Can't write article to file '%#'\n",$word);
			}
		}
	}
}

print _L("Total words processed: %#\n",$cards);

my $iCnt = safeExec(['ls', $images], ['wc', '-l']);
my $sCnt = safeExec(['ls', $sounds], ['wc', '-l']);
my $mCnt = safeExec(['ls', $videos], ['wc', '-l']);
my $cCnt = safeExec(['ls', $css], ['wc', '-l']);
$iCnt =~ s/\D//gs; 
$sCnt =~ s/\D//gs; 
$mCnt =~ s/\D//gs; 
$cCnt =~ s/\D//gs; 
print _L("Images: %#, Sounds: %#, Movies: %#, Styles: %#\n",$iCnt,$sCnt,$mCnt,$cCnt);
if (!$iCnt && !$sCnt && !$mCnt && !$cCnt)
{
	print _L("No multimedia files found. Removing Extensions directory.\n");
	system('rm', '-rf', $extensions);
}
else
{
	system('ln', '-s', '../Extensions', "$words/__E__");
}

print _L("Done\n");
system('open', $out) unless ($silent);
